@@ -9,7 +9,7 @@ use Crypt::Eksblowfish::Bcrypt ();
 use 5.008001;
 use utf8;
 
-our $VERSION = '1.201';
+our $VERSION = '1.202';
 $VERSION = eval $VERSION;
 
 sub add {
@@ -18,7 +18,7 @@ sub add {
     return $self;
 }
 
-sub bcrypt_b64digest { return Crypt::Eksblowfish::Bcrypt::en_base64(shift->_digest); }
+sub bcrypt_b64digest { return Crypt::Eksblowfish::Bcrypt::en_base64(shift->digest); }
 
 sub clone {
     my $self = shift;
@@ -31,42 +31,69 @@ sub clone {
 }
 
 sub cost {
-    my ($self, $cost) = @_;
-    if (defined $cost) {
-        $self->_check_cost($cost);
-        # bcrypt requires 2 digit costs, it dies if it's a single digit.
-        $self->{cost} = sprintf("%02d", $cost);
+    my $self = shift;
+    return $self->{cost} unless @_;
+
+    my $cost = shift;
+    # allow and undefined value to clear it
+    unless (defined $cost) {
+        delete $self->{cost};
         return $self;
     }
-    return $self->{cost};
+    $self->_check_cost($cost);
+    # bcrypt requires 2 digit costs, it dies if it's a single digit.
+    $self->{cost} = sprintf("%02d", $cost);
+    return $self;
 }
 
-sub digest { return shift->_digest; }
+sub digest {
+    my $self = shift;
+    $self->_check_cost;
+    $self->_check_salt;
+
+    my $hash = Crypt::Eksblowfish::Bcrypt::bcrypt_hash({
+        key_nul => 1,
+        cost    => $self->cost,
+        salt    => $self->salt,
+    }, $self->{_buffer});
+    $self->reset;
+    return $hash;
+}
 
 sub new {
     my $class = shift;
-
-    return bless {
+    my $self = bless {
         _buffer => '',
-    }, ref($class) || $class;
+    }, ref $class || $class;
+    return $self unless @_;
+    my $params = @_ > 1 ? {@_} : {%{$_[0]}};
+    $self->cost($params->{cost}) if $params->{cost};
+    $self->salt($params->{salt}) if $params->{salt};
+    return $self;
 }
 
 sub reset {
     my $self = shift;
-    delete $self->{_buffer};
+    $self->{_buffer} = '';
     delete $self->{cost};
     delete $self->{salt};
-    return $self->new;
+    return $self;
 }
 
 sub salt {
-    my ($self, $salt) = @_;
-    if (defined $salt) {
-        $self->_check_salt($salt);
-        $self->{salt} = $salt;
+    my $self = shift;
+    return $self->{salt} unless @_;
+
+    my $salt = shift;
+    # allow and undefined value to clear it
+    unless ( defined $salt ) {
+        delete $self->{salt};
         return $self;
     }
-    return $self->{salt};
+    # all other values go through the check
+    $self->_check_salt($salt);
+    $self->{salt} = $salt;
+    return $self;
 }
 
 # Checks that the cost is an integer in the range 1-31. Croaks if it isn't
@@ -87,21 +114,6 @@ sub _check_salt {
     }
 }
 
-# Returns the raw bcrypt digest and resets the object
-sub _digest {
-    my $self = shift;
-    $self->_check_cost;
-    $self->_check_salt;
-
-    my $hash = Crypt::Eksblowfish::Bcrypt::bcrypt_hash({
-        key_nul => 1,
-        cost    => $self->cost,
-        salt    => $self->salt,
-    }, $self->{_buffer});
-    $self->reset;
-    return $hash;
-}
-
 1;
 
 =encoding utf8
@@ -118,7 +130,7 @@ Digest::Bcrypt - Perl interface to the bcrypt digest algorithm
     use utf8;
     use Digest;   # via the Digest module (recommended)
 
-    my $bcrypt = Digest->new('Bcrypt');
+    my $bcrypt = Digest->new('Bcrypt', cost => 12, salt => 'abcdefgh♥stuff');
 
     # $cost is an integer between 1 and 31
     $bcrypt->cost(12);
@@ -149,6 +161,34 @@ L<Crypt::Eksblowfish::Bcrypt> library.
 Please note that you B<must> set a C<salt> of exactly 16 octets in length,
 and you B<must> provide a C<cost> in the range C<'1'..'31'>.
 
+=head1 ATTRIBUTES
+
+L<Digest::Bcrypt> implements the following attributes.
+
+=head2 cost
+
+    $bcrypt = $bcrypt->cost(20); # allows for method chaining
+    my $cost = $bcrypt->cost();
+
+An integer in the range C<'1'..'31'>, this is required.
+
+See L<Crypt::Eksblowfish::Bcrypt> for a detailed description of C<cost>
+in the context of the bcrypt algorithm.
+
+When called with no arguments, it will return the current cost.
+
+=head2 salt
+
+    $bcrypt = $bcrypt->salt('abcdefgh♥stuff'); # allows for method chaining
+    my $salt = $bcrypt->salt();
+
+Sets the value to be used as a salt. Bcrypt requires B<exactly> 16 octets of salt.
+
+It is recommenced that you use a module like L<Data::Entropy::Algorithms> to
+provide a truly randomized salt.
+
+When called with no arguments, it will return whatever is the current salt.
+
 =head1 METHODS
 
 L<Digest::Bcrypt> inherits all methods from L<Digest::base> and implements/overrides
@@ -156,12 +196,28 @@ the following methods as well.
 
 =head2 new
 
-    my $bcrypt = Digest->new('Bcrypt');
-    my $bcrypt = Digest::Bcrypt->new();
+    my $bcrypt = Digest->new('Bcrypt', %params);
+    my $bcrypt = Digest::Bcrypt->new(%params);
+    my $bcrypt = Digest->new('Bcrypt', \%params);
+    my $bcrypt = Digest::Bcrypt->new(\%params);
 
 Creates a new C<Digest::Bcrypt> object. It is recommended that you use the L<Digest>
 module in the first example rather than using L<Digest::Bcrypt> directly.
 
+Possible parameters are:
+
+=over
+
+=item cost
+
+An integer value between 1 and 31.
+
+=item salt
+
+A string of exactly 16 octets in length.
+
+=back
+
 =head2 add
 
     $bcrypt->add("a"); $bcrypt->add("b"); $bcrypt->add("c");
@@ -204,18 +260,6 @@ compatible with the standard MIME base64 encoding.
 
 Creates a clone of the C<Digest::Bcrypt> object, and returns it.
 
-=head2 cost
-
-    $bcrypt->cost($cost);
-    my $cost = $bcrypt->cost();
-
-An integer in the range C<'1'..'31'>, this is required.
-
-See L<Crypt::Eksblowfish::Bcrypt> for a detailed description of C<cost>
-in the context of the bcrypt algorithm.
-
-When called with no arguments, it will return the current cost.
-
 =head2 digest
 
     my $digest = $bcrypt->digest;
@@ -237,18 +281,6 @@ characters from the ranges C<'0'..'9'> and C<'a'..'f'>.
 
 Resets the object to the same internal state it was in when it was constructed.
 
-=head2 salt
-
-    $bcrypt->salt($salt);
-    my $salt = $bcrypt->salt();
-
-Sets the value to be used as a salt. Bcrypt requires B<exactly> 16 octets of salt.
-
-It is recommenced that you use a module like L<Data::Entropy::Algorithms> to
-provide a truly randomized salt.
-
-When called with no arguments, it will return whatever is the current salt.
-
 =head1 SEE ALSO
 
 L<Digest>, L<Crypt::Eksblowfish::Bcrypt>, L<Data::Entropy::Algorithms>
